@@ -104,34 +104,14 @@ class Grass7Algorithm(GeoAlgorithm):
         return self._icon
 
     def help(self):
-        localDoc = None
-        html = self.grass7Name + '.html'
-        if system.isWindows():
-            # For MS-Windows, use the configured GRASS7 path
-            localPath = os.path.join(Grass7Utils.grassPath(), 'docs/html', html)
-            if os.path.exists(localPath):
-                localDoc = os.path.abspath(localPath)
-        elif system.isMac():
-            # For MacOSX official package
-            localPath = os.path.join('/Applications/GRASS-7.0.app/Contents/MacOS/docs/html', html)
-            if os.path.exists(localPath):
-                localDoc = os.path.abspath(localPath)
+        helpPath = Grass7Utils.grassHelpPath()
+        if helpPath == '':
+            return False, None
+
+        if os.path.exists(helpPath):
+            return False, QUrl.fromLocalFile(os.path.join(helpPath, '{}.html'.format(self.grass7Name))).toString()
         else:
-            # For GNU/Linux distributions
-            searchPaths = ['/usr/share/doc/grass-doc/html', '/opt/grass/docs/html',
-                           '/usr/share/doc/grass/docs/html']
-            for path in searchPaths:
-                localPath = os.path.join(path, html)
-                if os.path.exists(localPath):
-                    localDoc = os.path.abspath(localPath)
-
-        # Found the local documentation
-        if localDoc:
-            localDoc = QUrl.fromLocalFile(localDoc).toString()
-            return False, localDoc
-
-        # Return the URL if local doc is not found
-        return False, 'http://grass.osgeo.org/grass70/manuals/' + self.grass7Name + '.html'
+            return False, helpPath + '{}.html'.format(self.grass7Name)
 
     def getParameterDescriptions(self):
         descs = {}
@@ -195,7 +175,7 @@ class Grass7Algorithm(GeoAlgorithm):
                     elif isinstance(output, OutputVector):
                         vectorOutputs += 1
                     if isinstance(output, OutputHTML):
-                        self.addOutput(OutputFile("rawoutput", output.description + 
+                        self.addOutput(OutputFile("rawoutput", output.description +
                                                   " (raw output)", "txt"))
                 line = lines.readline().strip('\n').strip()
             except Exception as e:
@@ -415,16 +395,16 @@ class Grass7Algorithm(GeoAlgorithm):
             if isinstance(param, (ParameterRaster, ParameterVector)):
                 value = param.value
                 if value in self.exportedLayers.keys():
-                    command += ' ' + param.name + '=' \
-                        + self.exportedLayers[value]
+                    command += ' ' + param.name + '="' \
+                        + self.exportedLayers[value] + '"'
                 else:
-                    command += ' ' + param.name + '=' + value
+                    command += ' ' + param.name + '="' + value + '"'
             elif isinstance(param, ParameterMultipleInput):
                 s = param.value
                 for layer in self.exportedLayers.keys():
                     s = s.replace(layer, self.exportedLayers[layer])
                 s = s.replace(';', ',')
-                command += ' ' + param.name + '=' + s
+                command += ' ' + param.name + '="' + s + '"'
             elif isinstance(param, ParameterBoolean):
                 if param.value:
                     command += ' ' + param.name
