@@ -445,7 +445,7 @@ int QgsWFSServer::getFeature( QgsRequestHandler& request, const QString& format
     QDomElement queryElem;
     for ( int i = 0; i < queryNodes.size(); i++ )
     {
-      queryElem = queryNodes.at( 0 ).toElement();
+      queryElem = queryNodes.at( i ).toElement();
       mTypeName = queryElem.attribute( "typeName", "" );
       if ( mTypeName.contains( ":" ) )
       {
@@ -455,7 +455,7 @@ int QgsWFSServer::getFeature( QgsRequestHandler& request, const QString& format
     }
     for ( int i = 0; i < queryNodes.size(); i++ )
     {
-      queryElem = queryNodes.at( 0 ).toElement();
+      queryElem = queryNodes.at( i ).toElement();
       mTypeName = queryElem.attribute( "typeName", "" );
       if ( mTypeName.contains( ":" ) )
       {
@@ -1913,7 +1913,10 @@ QString QgsWFSServer::createFeatureGeoJSON( QgsFeature* feat, int prec, QgsCoord
 
   QgsJSONExporter exporter;
   exporter.setSourceCrs( crs );
-  exporter.setPrecision( prec );
+  //QgsJSONExporter force transform geometry to ESPG:4326
+  //and the RFC 7946 GeoJSON specification recommends limiting coordinate precision to 6
+  Q_UNUSED( prec );
+  //exporter.setPrecision( prec );
 
   //copy feature so we can modify its geometry as required
   QgsFeature f( *feat );
@@ -1990,7 +1993,14 @@ QDomElement QgsWFSServer::createFeatureGML2( QgsFeature* feat, QDomDocument& doc
       delete centroid;
     }
     else
-      gmlElem = QgsOgcUtils::geometryToGML( geom, doc, prec );
+    {
+      QgsAbstractGeometryV2* abstractGeom = geom->geometry();
+      if ( abstractGeom )
+      {
+        gmlElem = abstractGeom->asGML2( doc, prec, "http://www.opengis.net/gml" );
+      }
+    }
+
     if ( !gmlElem.isNull() )
     {
       QgsRectangle box = geom->boundingBox();
@@ -2067,7 +2077,14 @@ QDomElement QgsWFSServer::createFeatureGML3( QgsFeature* feat, QDomDocument& doc
       delete centroid;
     }
     else
-      gmlElem = QgsOgcUtils::geometryToGML( geom, doc, "GML3", prec );
+    {
+      QgsAbstractGeometryV2* abstractGeom = geom->geometry();
+      if ( abstractGeom )
+      {
+        gmlElem = abstractGeom->asGML3( doc, prec, "http://www.opengis.net/gml" );
+      }
+    }
+
     if ( !gmlElem.isNull() )
     {
       QgsRectangle box = geom->boundingBox();
